@@ -2,54 +2,47 @@ condSmacof_matrix <- function (d, V, u.dim, W = NULL, it.max = 1000, gamma = 1e-
                                init = c('none', 'user'),
                                U.start, B.start)
 {
-  p <- u.dim
-  q <- dim(V)[2]
-  N <- nrow(as.matrix(d))
-  if (p > (N - 1 - q))
+  q <- ncol(V)
+  N <- nrow(V)
+  if (u.dim > (N - 1 - q))
     stop("Max u.dim is N-1-ncol(V)!")
 
-  tV <- t(V)
   one_n_t <- t(rep(1,N))
 
-  if (is.null(W)) {
-    W <- matrix(1, N, N)
-    W[is.na(as.matrix(d))] <- 0
-  }
-  w <- as.dist(W)
-  H <- diag(rowSums(W)) - W
-
-  G <- tV %*% H %*% V
-  Gp <- ginv(G)
-  GptV <- Gp %*% tV
-
   init <- match.arg(init, c('none', 'user'))
   if (init == 'none') {
     B.start <- diag(rep(1,q))
+    U.start <- matrix(runif(N * u.dim, min = -1), ncol = u.dim)
   } else {
-    if (is.null(B.start))
-      stop('B.start needs to be provided when init = "user"!')
+    if (is.null(B.start) | is.null(U.start)) {
+      stop('U.start and B.start need to be provided when init = "user"!')
+    } else {
+      B.start <- as.matrix(B.start)
+      U.start <- as.matrix(U.start)
+    }
   }
   B <- B.start
-
-  V.tilda <- V %*% B
-
-  if (init == 'none') {
-    U.start <- matrix(runif(N * p, min = -1), ncol = p)
-  } else {
-    U.start <- matrix(U.start)
-  }
   U <- U.start
 
+  if (is.null(W)) {
+    W <- matrix(1, N, N)
+    W[is.na(as.matrix(d))] <- 0
+    flag <- TRUE
+  } else {
+    flag <- FALSE
+  }
+  w <- as.dist(W)
+  H <- diag(rowSums(W)) - W
+  GptV <- ginv(t(V) %*% H %*% V) %*% t(V)
 
   eta.d <- sum(w*d^2)
+  V.tilda <- V %*% B
   dz <- condDist(U, V.tilda, one_n_t)
   sigma <- rep(0, it.max)
-  sigma[1] <- eta.d +
-    sum(diag(t(U)%*%H%*%U)) + sum(diag(t(V.tilda) %*% H %*% V.tilda)) -
-    2*sum(w*d*dz)
-
+  sigma[1] <- eta.d + sum(diag(t(U)%*%H%*%U)) +
+    sum(diag(t(V.tilda) %*% H %*% V.tilda)) - 2*sum(w*d*dz)
   gamma <- gamma*eta.d
-  if (is.null(W)) {
+  if (flag) {
     for (iter in 2:it.max) {
       cz <- -w*d/dz
       cz[which(cz == Inf)] <- 0
